@@ -54,7 +54,7 @@ struct _StatusbarItem
     gint position;
     
     gboolean mandatory;
-    gboolean conditional;
+    gboolean condition;
 };
 
 struct _StatusbarItemClass
